/*
 * BongoDB, an embedded document-based engine
 * Copyright (C) 2015 by Javier Sancho Fernandez <jsf at jsancho dot org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __BONGODB_H_
#define __BONGODB_H_


/* BongoDB Client */

typedef struct {
  char *key;
  char *value;
  struct bongo_record_tag_t *next;
} bongo_record_tag_t;

typedef struct {
  bongo_record_tag_t *tags;
  struct bongo_record_node_t *next;
} bongo_record_node_t;

typedef struct {
  const char *col_name;
  int n_records;
  bongo_record_node_t *records;
  struct bongo_collection_node_t *next;
} bongo_collection_node_t;

typedef struct {
  const char *db_name;
  int n_collections;
  bongo_collection_node_t *collections;
  struct bongo_database_node_t *next;
} bongo_database_node_t;

typedef struct {
  int n_databases;
  bongo_database_node_t *databases;
} bongo_client_t;

bongo_client_t *
bongo_client_new (void);

bongo_database_node_t *
bongo_database_node_new (bongo_client_t *client,
			 const char *db_name);

bongo_collection_node_t *
bongo_collection_node_new (bongo_database_node_t *database_node,
			   const char *col_name);

bongo_database_node_t *
bongo_database_node_search (bongo_client_t *client,
			    const char *db_name);

bongo_collection_node_t *
bongo_collection_node_search (bongo_database_node_t *database_node,
			      const char *col_name);

void
bongo_collection_node_destroy (bongo_collection_node_t *collection_node);

void
bongo_database_node_destroy (bongo_database_node_t *database_node);

void
bongo_client_destroy (bongo_client_t *client);


/* BongoDB Database */

typedef struct {
  bongo_database_node_t *database_node;
} bongo_database_t;

bongo_database_t *
bongo_client_get_database (bongo_client_t *client,
			   const char *db_name);

void
bongo_database_destroy (bongo_database_t *database);


/* BongoDB Collection */

typedef struct {
  bongo_collection_node_t *collection_node;
} bongo_collection_t;

bongo_collection_t *
bongo_client_get_collection (bongo_client_t *client,
			     const char *db_name,
			     const char *col_name);

bool
bongo_collection_insert (bongo_collection_t *collection,
			 bongo_insert_flags_t flags,
			 const char *document,
			 const bongo_write_concern_t *write_concern,
			 bongo_error_t *error);

void
bongo_collection_destroy (bongo_collection_t *collection);


#endif /* __BONGODB_H_ */
